package com.kim.apidoc.core.common.description;

import com.github.javaparser.ast.comments.Comment;
import com.github.javaparser.javadoc.Javadoc;
import com.google.common.collect.Lists;
import com.kim.apidoc.core.common.helper.CommentHelper;
import com.kim.apidoc.core.common.helper.StringHelper;
import com.kim.apidoc.core.schema.Row;
import lombok.Getter;
import lombok.Setter;

import java.util.Collection;

/**
 * The type Type description.
 */
@Setter
@Getter
public abstract class TypeDescription {

    /**
     * The Prefix.
     */
    protected String prefix = "";
    /**
     * The Key.
     */
    protected String key = "";
    /**
     * The Type.
     */
    protected String type;
    /**
     * The Condition.
     */
    protected StringBuilder condition = new StringBuilder();
    /**
     * 说明.
     */
    protected String remark;
    /**
     * 值.
     */
    protected Object value;
    /**
     * 默认值.
     */
    protected Object defaultValue;
    /**
     * 是否必填.
     */
    protected Boolean required;

    /**
     * Is available boolean.
     *
     * @return the boolean
     */
    public boolean isAvailable() {
        return !isUnAvailable();
    }

    /**
     * Is un available boolean.
     *
     * @return the boolean
     */
    public boolean isUnAvailable() {
        return this instanceof UnAvailableTypeDescription;
    }

    /**
     * Is primitive boolean.
     *
     * @return the boolean
     */
    public boolean isPrimitive() {
        return this instanceof PrimitiveTypeDescription;
    }

    /**
     * As primitive primitive type description.
     *
     * @return the primitive type description
     */
    public PrimitiveTypeDescription asPrimitive() {
        return (PrimitiveTypeDescription) this;
    }

    /**
     * Is string boolean.
     *
     * @return the boolean
     */
    public boolean isString() {
        return this instanceof StringTypeDescription;
    }

    /**
     * As string string type description.
     *
     * @return the string type description
     */
    public StringTypeDescription asString() {
        return (StringTypeDescription) this;
    }

    /**
     * Is array boolean.
     *
     * @return the boolean
     */
    public boolean isArray() {
        return this instanceof ArrayTypeDescription;
    }

    /**
     * As array array type description.
     *
     * @return the array type description
     */
    public ArrayTypeDescription asArray() {
        return (ArrayTypeDescription) this;
    }

    /**
     * Is object boolean.
     *
     * @return the boolean
     */
    public boolean isObject() {
        return this instanceof ObjectTypeDescription;
    }

    /**
     * As object object type description.
     *
     * @return the object type description
     */
    public ObjectTypeDescription asObject() {
        return (ObjectTypeDescription) this;
    }

    /**
     * Add remark.
     *
     * @param value the value
     */
    public void addRemark(String value) {
        if (value == null) {
            return;
        }
        if (remark == null) {
            remark = value;
        } else {
            remark += " " + value;
        }
    }

    /**
     * Full key string.
     *
     * @return the string
     */
    public String fullKey() {
        return StringHelper.join(".", prefix, key);
    }

    /**
     * Rows collection.
     *
     * @return the collection
     */
    public Collection<Row> rows(String requestParameterType) {
        String fullKey = fullKey();
        if (StringHelper.isBlank(fullKey)) {
            return Lists.newArrayList();
        }
        String def;
        if (defaultValue != null) {
            def = String.valueOf(defaultValue);
        } else if (value != null) {
            def = String.valueOf(value);
        } else {
            def = "";
        }

        if (required != null) {
            condition.append("required=").append(required);
        }

        return Lists.newArrayList(new Row(key, type, false, condition.toString(), def, remark, requestParameterType));
    }


    /**
     * Rows collection.
     *
     * @return the collection
     */
    public Collection<Row> rows() {
        return this.rows(null);
    }

    public void accept(Comment comment) {
        if (!comment.isJavadocComment()) {
            setRemark(comment.getContent());
            return;
        }
        Javadoc javadoc = comment.asJavadocComment().parse();
        setRemark(CommentHelper.getDescription(javadoc.getDescription()));
    }

}
