package com.kim.apidoc.core.schema;

import com.kim.apidoc.core.common.ObjectMappers;
import com.kim.apidoc.core.common.QueryStringBuilder;
import com.kim.apidoc.core.common.helper.StringHelper;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ObjectNode;
import lombok.Getter;
import lombok.Setter;

import java.util.*;

/**
 * 小节，一个请求，封装为一个小节
 *
 * @author fengyuchenglun
 * @version 1.0.0
 */
@Setter
@Getter
public class Section extends Node {

    /**
     * 请求方法.
     */
    Method method;
    /**
     * 请求url.
     */
    String uri;
    /**
     * 请求头.
     */
    Map<String, Header> inHeaders = new LinkedHashMap<>();
    /**
     * 路径变量.
     */
    ObjectNode pathVariable = ObjectMappers.instance.createObjectNode();
    /**
     * The Parameter.
     */
    JsonNode parameter;
    /**
     * The Query parameter.
     */
    boolean queryParameter = true;

    /**
     * The Request rows.
     */
    Map<String, Row> requestRows = new LinkedHashMap<>();

    /**
     * The Out headers.
     */
    Map<String, Header> outHeaders = new LinkedHashMap<>();
    /**
     * The Response.
     */
    JsonNode response;
    /**
     * The Response rows.
     */
    Map<String, Row> responseRows = new LinkedHashMap<>();
    /**
     * The Raw response.
     */
    Object rawResponse;


    /**
     * Add request row.
     *
     * @param row the row
     */
    public void addRequestRow(Row row) {
        requestRows.put(row.getKey(), row);
    }

    /**
     * Add request rows.
     *
     * @param rows the rows
     */
    public void addRequestRows(Collection<Row> rows) {
        for (Row row : rows) {
            if (row.getKey() != null && !requestRows.containsKey(row.getKey())) {
                requestRows.put(row.getKey(), row);
            }
        }
    }

    /**
     * Get request line string.
     *
     * @return the string
     */
    public String getRequestLine() {
        StringBuilder builder = new StringBuilder(this.method.toString());
        builder.append(" ").append(this.uri);
        if (this.queryParameter && Objects.equals("GET", this.method)) {
            String parameterString = getParameterString();
            if (StringHelper.nonBlank(parameterString)) {
                builder.append("?").append(parameterString);
            }
        }
        builder.append(" HTTP/1.1");
        return builder.toString();
    }

    /**
     * Get parameter string string.
     *
     * @return the string
     */
    public String getParameterString() {
        if (queryParameter && parameter instanceof ObjectNode) {
            return new QueryStringBuilder().append((ObjectNode) parameter).toString();
        }
        return ObjectMappers.pretty(parameter);
    }

    /**
     * Has request body boolean.
     *
     * @return the boolean
     */
    public boolean hasRequestBody() {
        if (Objects.equals("GET", this.method)) {
            return false;
        }
        return parameter != null && parameter.size() > 0;
    }

    /**
     * Add response row.
     *
     * @param row the row
     */
    public void addResponseRow(Row row) {
        responseRows.put(row.getKey(), row);
    }

    /**
     * Add response rows.
     *
     * @param rows the rows
     */
    public void addResponseRows(Collection<Row> rows) {
        for (Row row : rows) {
            if (row.getKey() != null && !responseRows.containsKey(row.getKey())) {
                responseRows.put(row.getKey(), row);
            }
        }
    }

    /**
     * Has response body boolean.
     *
     * @return the boolean
     */
    public boolean hasResponseBody() {
        return response != null || rawResponse != null;
    }

    /**
     * Get response string string.
     *
     * @return the string
     */
    public String getResponseString() {
        if (response != null) {
            return ObjectMappers.pretty(response);
        }
        return String.valueOf(rawResponse);
    }

    /**
     * Add in header.
     *
     * @param header the header
     */
    public void addInHeader(Header header) {
        if (!inHeaders.containsKey(header.getKey())) {
            inHeaders.put(header.getKey(), header);
        }
    }

    /**
     * Add out header.
     *
     * @param header the header
     */
    public void addOutHeader(Header header) {
        if (!outHeaders.containsKey(header.getKey())) {
            outHeaders.put(header.getKey(), header);
        }
    }

}
